import logging

from isc_common import setAttr, delAttr, Wrapper
from isc_common.http.RPCRequest import RPCRequest
from isc_common.json import BytesToJson

logger = logging.getLogger(__name__)


class RequestData:
    def __init__(self, _dict=None, request=None):
        if request:
            self.__data_dict__ = request.get_data()
        elif _dict:
            self.__data_dict__ = _dict
        else:
            self.__data_dict__ = dict()
        super()

    def getDataOfKey(self, key, default=None):
        return self.__data_dict__.get(key, default)

    def update(self, key, value):
        self.__data_dict__.update((key, value))
        return RequestData(self.__data_dict__)

    def getDataWithOutField(self, list_fields):
        if isinstance(list_fields, list):
            return RequestData({key: self.__data_dict__[key] for key in self.__data_dict__ if not key in list_fields})
        else:
            return RequestData(self.__data_dict__)

    def getDataNotContain(self, *sub_strs):
        res = RequestData()
        for sub in sub_strs:
            self.__data_dict__ = {key: self.__data_dict__[key] for key in self.__data_dict__ if not str(key).__contains__(sub)}
        return RequestData(self.__data_dict__)

    def dict(self):
        return self.__data_dict__


class DSRequest(RPCRequest):
    alive_only = None
    begdate = None
    dataPageSize = 75
    drawAheadRatio = 1.2
    enabledAll = None
    endRow = None
    enddate = None
    httpHeaders = None
    json = None
    startRow = None
    tag = None
    transaction = None
    visibleMode = 'none'

    _user = None

    @property
    def user(self):
        from isc_common.auth.models.user import User
        if self._user is None:
            if self.httpHeaders is not None and self.httpHeaders.USER_ID:
                self._user = User.objects.get(id=self.httpHeaders.USER_ID)
                return self._user
            else:
                if self.transaction is not None:
                    operations = self.transaction.operations
                    if isinstance(operations, list):
                        for operation in operations:
                            data = operation.data
                            if data.httpHeaders is not None and data.httpHeaders.USER_ID is not None:
                                self._user = User.objects.get(id=data.httpHeaders.USER_ID)
                                return self._user

                # raise Exception('user not enable type')
        return self._user

    @property
    def user_id(self):
        return self.user.id

    @property
    def fio(self):
        return self.user.get_short_name

    @property
    def username(self):
        return self.user.username

    @property
    def is_admin(self):
        return self.user.is_admin

    @property
    def is_develop(self):
        return self.user.is_develop

    def set_user_data(self, user):
        from isc_common.auth.models.user import User
        from isc_common.number import ToInt

        _user = ToInt(user)
        if isinstance(_user, int):
            self._user = User.objects.get(id=_user)
        elif isinstance(user, str):
            self._user = User.objects.get(username=user)
        else:
            raise Exception('user not enable type')

    def __init__(self, request):
        from django.http import QueryDict
        if request is not None and request.body != b'':
            self.json = BytesToJson(request.body)
            if isinstance(self.json, dict) and len(self.json) > 0:
                data = self.json.get('data')
                if isinstance(data, dict):
                    httpHeaders = data.get('httpHeaders')
                    if isinstance(httpHeaders, dict):

                        if httpHeaders.get('WS_CHANNEL') is not None:
                            self.ws_channel = httpHeaders.get('WS_CHANNEL')

                        if httpHeaders.get('WS_PORT') is not None:
                            self.ws_port = httpHeaders.get('WS_PORT')

                        if httpHeaders.get('HOST') is not None:
                            self.host = httpHeaders.get('HOST')

                        if httpHeaders.get('USERNAME') is not None:
                            self.set_user_data(user=httpHeaders.get('USERNAME'))

                        if httpHeaders.get('USER_ID') is not None:
                            self.set_user_data(user=httpHeaders.get('USER_ID'))

                        delAttr(self.json.get('data'), 'httpHeaders')
                    else:
                        if data.get('user_id') is not None:
                            self.set_user_data(user=data.get('user_id'))
                else:
                    login = self.json.get('login')
                    if login is not None:
                        self.set_user_data(user=login)
                    else:
                        httpHeaders = self.json.get('httpHeaders')
                        if httpHeaders is not None:
                            self.set_user_data(user=httpHeaders.get('USER_ID'))

        elif isinstance(request.GET, QueryDict):
            get_dict = dict(request.GET)
            ws_channel = get_dict.get('ws_channel')
            if isinstance(ws_channel, list) and len(ws_channel) > 0:
                self.ws_channel = ws_channel[0]

            port = get_dict.get('port')
            if isinstance(port, list) and len(port) > 0:
                self.ws_port = port[0]

            host = get_dict.get('host')
            if isinstance(host, list) and len(host) > 0:
                self.host = host[0]

            user_id = get_dict.get('user_id')
            if isinstance(user_id, list) and len(user_id) > 0:
                self.set_user_data(user=user_id[0])

        RPCRequest.__init__(self, self.json)

    def delete_underscore_element(self, data):
        if not isinstance(data, dict):
            raise Exception('data must be dict')

        res = dict()
        for key, value in data.items():
            if str(key).find('_', 0, 1) == -1:
                setAttr(res, key, value)
        return res

    def get_data(self, excluded_keys=['grid']):
        res = None
        if isinstance(self.json, dict):
            data = self.json.get('data')
            if isinstance(data, dict):
                res = data
            elif isinstance(data, str):
                return data
            else:
                res = self.json

            # res = dict((key, value) for (key, value) in res.items() if not key.startswith('_')  and key not in excluded_keys)
            res = dict((key, value) for (key, value) in res.items() if key not in excluded_keys)
        # return self.delete_underscore_element(res)
        return res

    def get_data_wrapped(self, excluded_keys=['grid'], cls=Wrapper):
        return cls(**self.get_data(excluded_keys=excluded_keys))

    def get_old_data(self, excluded_keys=['grid']):
        res = None
        if isinstance(self.json, dict):
            data = self.json.get('oldValues')
            if isinstance(data, dict):
                res = data
            elif isinstance(data, str):
                return data
            else:
                res = self.json

            res = dict((key, value) for (key, value) in res.items() if not key.startswith('_') and key not in excluded_keys)
        # return self.delete_underscore_element(res)
        return res

    def get_old_data_wrapped(self, excluded_keys=['grid'], cls=Wrapper):
        return cls(**self.get_old_data(excluded_keys=excluded_keys))

    def get_all_data(self, excluded_keys=['grid']):
        res = None
        if isinstance(self.json, dict):
            data = self.json.get('data')
            if isinstance(data, dict):
                res = data
            elif isinstance(data, str):
                return data
            else:
                res = self.json

        return res

    def set_data(self, data):
        if isinstance(data, dict):
            setAttr(self.json, 'data', data)
        else:
            delAttr(self.json, 'data')

    def update_data(self, data):
        if isinstance(data, dict):
            self.json.update(data)

    def set_sort_by(self, sort_by):
        if not isinstance(sort_by, list):
            sort_by = [sort_by]

        setAttr(self.json, 'sortBy', sort_by)

    def set_criteria(self, criteria, excluded_keys=['grid']):
        data = self.get_all_data(excluded_keys=excluded_keys)
        if isinstance(data, dict):
            setAttr(data, 'criteria', criteria)
        else:
            delAttr(data, 'criteria')

    def get_criteria(self, excluded_keys=['grid']):
        data = self.get_data(excluded_keys=excluded_keys)
        if isinstance(data, dict):
            criteria = data.get('criteria')
        else:
            return dict()

        if isinstance(criteria, list):
            return criteria
        else:
            return []

    def get_data_array(self, excluded_keys=['grid']):
        res = None
        if isinstance(self.json, dict):
            transaction = self.json.get('transaction')
            if transaction:
                operations = transaction.get('operations')
            else:
                return (False, [self.get_data()])

            res = (True, [dict((key, value) for (key, value) in operation.items() if not str(key).startswith('_') and key not in excluded_keys) for operation in operations])
            # res = (True, [dict((key, value) for (key, value) in operation.items() if value is not None and not str(key).startswith('_') and key not in excluded_keys) for operation in operations])
        return res

    def get_oldValues(self, excluded_keys=['grid']):
        res = None
        if isinstance(self.json, dict):
            if self.json.get('oldValues', None) and isinstance(self.json.get('oldValues', None), dict):
                res = self.json.get('oldValues')
            else:
                res = self.json

            res = dict((key, value) for (key, value) in res.items() if not str(key).startswith('_') and key not in excluded_keys)
            # res = dict((key, value) for (key, value) in res.items() if value is not None and not str(key).startswith('_') and key not in excluded_keys)
        return res

    def get_id(self):
        return self.get_data().get('id', self.get_oldValues().get('id'))

    def get_operationtype(self):
        return self.json.get('operationType')

    def get_username(self):
        return self.get_data().get('username')

    def get_ids(self, field_name='id'):
        multi, data_array = self.get_data_array()
        if data_array:
            if multi:
                ids = [record.get('data').get(field_name) for record in data_array]
            else:
                ids = [record.get('data').get(field_name) if record.get('data') else record.get(field_name) for record in data_array]
        else:
            ids = [0]
        return [id for id in ids if id is not None]

    def get_tuple_ids(self, field_name='id'):
        multi, data_array = self.get_data_array()
        if data_array:
            if multi:
                ids = [(record.get('data').get(field_name), record.get('visibleMode', 'none')) for record in data_array]
            else:
                ids = [(record.get('data').get(field_name), self.visibleMode) if record.get('data') else (record.get(field_name), self.visibleMode) for record in data_array]
        else:
            ids = [(0, "none")]
        return ids

    def get_olds_tuple_ids(self, field_name='id'):
        multi, data_array = self.get_data_array()
        if data_array:
            if multi:
                ids = [(record.get('oldValues').get(field_name), record.get('visibleMode', 'none')) for record in data_array]
            else:
                ids = [(record.get('oldValues').get(field_name), self.visibleMode) if record.get('data') else (record.get(field_name), self.visibleMode) for record in data_array]
        else:
            ids = [(0, "none")]
        return ids

    def get_old_ids(self, field_name='id'):
        multi, data_array = self.get_data_array()
        if data_array:
            if multi:
                ids = [record.get('oldValues').get(field_name) for record in data_array]
            else:
                ids = [record.get('oldValues').get(field_name) if record.get('oldValues') else record.get(field_name) for record in data_array]
        else:
            ids = [0]
        return [id for id in ids if id is not None]

    def __str__(self):
        return str(self.__dict__)



from django.urls import path

from react.views import fragment_params_view

urlpatterns = [

    path('Fragment_params_view/Fetch/', fragment_params_view.Fragment_params_view_Fetch),
    path('Fragment_params_view/Add', fragment_params_view.Fragment_params_view_Add),
    path('Fragment_params_view/Update', fragment_params_view.Fragment_params_view_Update),
    path('Fragment_params_view/Remove', fragment_params_view.Fragment_params_view_Remove),
    path('Fragment_params_view/Lookup/', fragment_params_view.Fragment_params_view_Lookup),
    path('Fragment_params_view/Info/', fragment_params_view.Fragment_params_view_Info),
    path('Fragment_params_view/Copy', fragment_params_view.Fragment_params_view_Copy),

]



import logging

from bitfield import BitField
from django.db.models import CheckConstraint , F , Q
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager , CommonManagetWithLookUpFieldsBaseRefQuerySet
from isc_common.models.base_ref import BaseRefHierarcy
from isc_common.models.tree_audit import TreeAuditModelManager
from react.models.page_level_types import Page_level_types

logger = logging.getLogger( __name__ )


class PagesQuerySet( CommonManagetWithLookUpFieldsBaseRefQuerySet ) :
    pass


class PagesManager( CommonManagetWithLookUpFieldsManager ) :
    def generateFromRequest( self , request ) :
        from react.models.generators import TypeScriptGenerator

        request = DSRequest( request=request )
        data = request.get_data()

        TypeScriptGenerator().generate( data=data , user=request.user )

        return data

    @classmethod
    def props( cls ) :
        return BitField( flags=(
            ('application' , 'Приложение') ,  # 1
        ) , default=0 , db_index=True )

    @classmethod
    def getRecord( cls , record ) :
        res = {
            'code' : record.code ,
            'deliting' : record.deliting ,
            'description' : record.description ,
            'editing' : record.editing ,
            'id' : record.id ,
            'name' : record.name ,
            'parent_id' : record.parent.id if record.parent else None ,
            'type__name' : record.type.name ,
            'type_id' : record.type.id ,
        }
        return res

    def get_queryset( self ) :
        return PagesQuerySet( self.model , using=self._db )


class Pages( BaseRefHierarcy ) :
    type = ForeignKeyProtect( Page_level_types )
    props = PagesManager.props()

    objects = PagesManager()
    tree_objects = TreeAuditModelManager()

    def __str__( self ) :
        return f'ID:{self.id}, code: {self.code}, name: {self.name}, description: {self.description}, type: [{self.type}]'

    def __repr__( self ) :
        return self.__str__()

    class Meta :
        verbose_name = 'Структура страницы'
        constraints = [
            CheckConstraint( check=~Q( id=F( 'parent_id' ) ) , name=f'c_Pages' ) ,
        ]



import logging

from django.db import connection
from django.db.models import Model

logger = logging.getLogger( __name__ )


class Model_links( Model ) :

    @classmethod
    def get_link( cls , model_id , model_code , model , model_link , model_link_fk ) :
        with connection.cursor() as cursor :
            cursor.execute( f'''select cml.id, cml.link
                                from {model} as lg
                                join {model_link} as lgl on lg.id = lgl.{model_link_fk}
                                join isc_common_links as cml on lgl.link_id = cml.id
                                where lg.id = %s
                                and lgl.code = %s''' , [ model_id , model_code ] )
            row = cursor.fetchone()
            if row is None :
                return None , None
            else :
                id , link = row
            return id , link

    @classmethod
    def update_or_create_link( cls , main_model , link_field_name , link , pk_name='id' ) :
        from isc_common.models.links import Links

        if link != '' or link is not None :
            if eval( f'main_model.{link_field_name} is None' , dict() , dict( main_model=main_model ) ) :
                res = eval( f'links.objects.create(link=link)' , dict() , dict( link=link , links=Links ) )
            else :
                res , _ = eval( f'links.objects.update_or_create(id=main_model.{link_field_name}.id, defaults=dict(link=link))' , dict() , dict( link=link , links=Links , main_model=main_model ) )

            eval( f'main_model._meta.concrete_model.objects.filter({pk_name}=main_model.{pk_name}).update({link_field_name}=res)' , dict() , dict( res=res , main_model=main_model ) )

    def __str__( self ) :
        return f'ID:{self.id}'

    def __repr__( self ) :
        return self.__str__()

    class Meta :
        verbose_name = 'Базовый класс'
        abstract = True



# Generated by Django 2.1.7 on 2019-02-16 23:17

from django.db import migrations
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0005_usergroup_permission'),
    ]

    operations = [
        migrations.AlterField(
            model_name='widgets_trees',
            name='id_widget',
            field=isc_common.fields.code_field.CodeField(unique=True),
        ),
    ]



import logging
import os
import shutil
from datetime import datetime
from urllib.parse import urljoin

from django.conf import settings
from django.core.files import File
from django.core.files.storage import Storage
from django.core.signals import setting_changed
from django.utils import timezone
from django.utils._os import safe_join
from django.utils.deconstruct import deconstructible
from django.utils.encoding import filepath_to_uri
from django.utils.functional import cached_property
from isc_common.common import SFTP, FILES_STORE_UNKNOWN_TYPE
from isc_common.oss.functions import get_tmp_dir

logger = logging.getLogger(__name__)


@deconstructible
class CryptoFileSystemStorage(Storage):

    def __init__(self, location=None, base_url=None, file_permissions_mode=None, directory_permissions_mode=None):
        self._location = location
        self._base_url = base_url
        self._file_permissions_mode = file_permissions_mode
        self._directory_permissions_mode = directory_permissions_mode
        setting_changed.connect(self._clear_cached_properties)

    def _clear_cached_properties(self, setting, **kwargs):
        """Reset setting based property values."""
        if setting == 'FILES_STORE':
            self.__dict__.pop('base_location', None)
            self.__dict__.pop('location', None)
        elif setting == 'MEDIA_URL':
            self.__dict__.pop('base_url', None)
        elif setting == 'FILE_UPLOAD_PERMISSIONS':
            self.__dict__.pop('file_permissions_mode', None)
        elif setting == 'FILE_UPLOAD_DIRECTORY_PERMISSIONS':
            self.__dict__.pop('directory_permissions_mode', None)

    def _value_or_setting(self, value, setting):
        return setting if value is None else value

    @cached_property
    def base_location(self):
        if isinstance(settings.FILES_STORE, str):
            file_store = settings.FILES_STORE
        elif isinstance(settings.FILES_STORE, dict) and settings.FILES_STORE.get('mode') == SFTP:
            file_store = get_tmp_dir()
        else:
            raise Exception(FILES_STORE_UNKNOWN_TYPE)

        return self._value_or_setting(self._location, file_store)

    @cached_property
    def location(self):
        return os.path.abspath(self.base_location)

    @cached_property
    def base_url(self):
        if self._base_url is not None and not self._base_url.endswith('/'):
            self._base_url += '/'
        return self._value_or_setting(self._base_url, settings.MEDIA_URL)

    @cached_property
    def file_permissions_mode(self):
        return self._value_or_setting(self._file_permissions_mode, settings.FILE_UPLOAD_PERMISSIONS)

    @cached_property
    def directory_permissions_mode(self):
        return self._value_or_setting(self._directory_permissions_mode, settings.FILE_UPLOAD_DIRECTORY_PERMISSIONS)

    def _open(self, name, mode='rb'):
        return File(open(self.path(name), mode))

    def _save(self, name, content):
        full_path = self.path(name)

        if isinstance(settings.FILES_STORE, str):
            _dir, _filename = os.path.split(full_path)
            _filename1, _ = _filename.split('_')

            name = f'{os.path.abspath(settings.FILES_STORE)}{os.sep}{_filename}'

            shutil.copy2(src=f'{_dir}{os.sep}{_filename1}', dst=name)

            if self.file_permissions_mode is not None:
                os.chmod(name, self.file_permissions_mode)

        elif isinstance(settings.FILES_STORE, dict) and settings.FILES_STORE.get('mode') == SFTP:
            _dir, _filename = os.path.split(full_path)
            _filename1, _ = _filename.split('_')
            name = f'{settings.FILES_STORE.get("PATH")}{os.sep}{_filename}'
            localpath = f'{_dir}{os.sep}{_filename1}'

            ssh_client = settings.SSH_CLIENTS.client(settings.FILES_STORE)
            res = ssh_client.put(localpath=localpath, remotepath=name)
            if res is not None:
                raise res

            if self.file_permissions_mode is not None:
                ssh_client.chmod(path=name, mode=self.file_permissions_mode)
        else:
            raise Exception(FILES_STORE_UNKNOWN_TYPE)

        # Store filenames with forward slashes, even on Windows.
        return name.replace('\\', '/')

    def delete(self, name):
        assert name, "The name argument is not allowed to be empty."
        name = self.path(name)
        # If the file or directory exists, delete it from the filesystem.
        try:
            if os.path.isdir(name):
                os.rmdir(name)
            else:
                os.remove(name)
        except FileNotFoundError as e:
            raise e
            # logger.warning(f'{name}: not found.')

    def exists(self, name):
        return os.path.exists(self.path(name))

    def listdir(self, path):
        path = self.path(path)
        directories, files = [], []
        for entry in os.listdir(path):
            if os.path.isdir(os.path.join(path, entry)):
                directories.append(entry)
            else:
                files.append(entry)
        return directories, files

    def path(self, name):
        return safe_join(self.location, name)

    def size(self, name):
        return os.path.getsize(self.path(name))

    def url(self, name):
        if self.base_url is None:
            raise ValueError("This file is not accessible via a URL.")
        url = filepath_to_uri(name)
        if url is not None:
            url = url.lstrip('/')
        return urljoin(self.base_url, url)

    def _datetime_from_timestamp(self, ts):
        if settings.USE_TZ:
            # Safe to use .replace() because UTC doesn't have DST
            return datetime.utcfromtimestamp(ts).replace(tzinfo=timezone.utc)
        else:
            return datetime.fromtimestamp(ts)

    def get_accessed_time(self, name):
        return self._datetime_from_timestamp(os.path.getatime(self.path(name)))

    def get_created_time(self, name):
        return self._datetime_from_timestamp(os.path.getctime(self.path(name)))

    def get_modified_time(self, name):
        return self._datetime_from_timestamp(os.path.getmtime(self.path(name)))



from django.urls import path
from permission.views import widgets_trees

urlpatterns = [

    path( 'Widgets_trees/Fetch/' , widgets_trees.Widgets_trees_Fetch ) ,
    path( 'Widgets_trees/Add' , widgets_trees.Widgets_trees_Add ) ,
    path( 'Widgets_trees/Update' , widgets_trees.Widgets_trees_Update ) ,
    path( 'Widgets_trees/Remove' , widgets_trees.Widgets_trees_Remove ) ,
    path( 'Widgets_trees/Lookup/' , widgets_trees.Widgets_trees_Lookup ) ,
    path( 'Widgets_trees/Info/' , widgets_trees.Widgets_trees_Info ) ,

]



# Generated by Django 3.2.6 on 2021-08-22 16:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0104_alter_images_unique_together'),
    ]

    operations = [
        migrations.AlterField(
            model_name='images',
            name='real_name',
            field=models.TextField(db_index=True, verbose_name='Первоначальное имя файла'),
        ),
    ]



from django.urls import path

from tracker.views import messages_whom

urlpatterns = [

    path('Messages_whom/Fetch/', messages_whom.Messages_whom_Fetch),
    path('Messages_whom/Add', messages_whom.Messages_whom_Add),
    path('Messages_whom/Update', messages_whom.Messages_whom_Update),
    path('Messages_whom/Remove', messages_whom.Messages_whom_Remove),
    path('Messages_whom/Lookup/', messages_whom.Messages_whom_Lookup),
    path('Messages_whom/Info/', messages_whom.Messages_whom_Info),

]



import logging

from django.db import transaction

from isc_common.http.DSResponse import JsonResponseWithException
from isc_common.models.upload_image import Common_UploadImage
from isc_common.models.users_images import Users_images
from lfl_admin.user_ext.models.administrators import Administrators

logger = logging.getLogger(__name__)


@JsonResponseWithException()
class DSResponse_Fragment_params_UploadImage(Common_UploadImage):
    def __init__(self, request):
        from react.models.fragment_params import Fragment_params

        with transaction.atomic():
            file = request.FILES.get('upload_attatch')
            if file is not None:
                image, fragment_id = self.upload_image(request=request)
                res, created = Fragment_params.objects.update_or_create(fragment_id=fragment_id, defaults=dict(image=image))
                logger.debug(f'Created: {created}')



from django.urls import path

from isc_common.views import image_types

urlpatterns = [

    path('Image_types/Fetch/', image_types.Image_types_Fetch),
    path('Image_types/Add', image_types.Image_types_Add),
    path('Image_types/Update', image_types.Image_types_Update),
    path('Image_types/Remove', image_types.Image_types_Remove),
    path('Image_types/Lookup/', image_types.Image_types_Lookup),
    path('Image_types/Info/', image_types.Image_types_Info),
    path('Image_types/Copy', image_types.Image_types_Copy),

]



import logging

from django.db.models import TextField, Model, BigAutoField

logger = logging.getLogger(__name__)


class Debug(Model):
    id = BigAutoField(primary_key=True, verbose_name="Идентификатор")
    params = TextField(null=True, blank=True)
    text = TextField()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Отладочная информация'



# Generated by Django 3.1.7 on 2021-03-26 09:49

from django.db import migrations

import isc_common.fields.code_field


class Migration(migrations.Migration):
    dependencies = [
        ('refs', '0011_auto_20210326_0948'),
    ]

    operations = [
        migrations.AlterField(
            model_name='type_param',
            name='code',
            field=isc_common.fields.code_field.CodeStrictField(unique=True),
        ),
    ]



# Generated by Django 3.2 on 2021-04-11 16:06

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('isc_common', '0094_alter_debug_id'),
    ]

    operations = [
        migrations.AddField(
            model_name='users_images',
            name='old_id',
            field=models.BigIntegerField(blank=True, db_index=True, null=True, unique=True),
        ),
    ]



# Generated by Django 3.2.6 on 2021-08-22 15:08

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0103_users_old_images'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='images',
            unique_together={('real_name', 'size', 'image_type')},
        ),
    ]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from isc_common.models.images import Images, ImagesManager
from isc_common.models.upload_image import DSResponse_UploadImage


@JsonResponseWithException()
def Images_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Images.objects.
                select_related().
                get_range_rows1(
                request=request,
                function=ImagesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Images_Add(request):
    return JsonResponse(DSResponseAdd(data=Images.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Images_Update(request):
    return JsonResponse(DSResponseUpdate(data=Images.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Images_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Images.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Images_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Images.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Images_Info(request):
    return JsonResponse(DSResponse(request=request, data=Images.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Images_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Images.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Images_Upload(request):
    return JsonResponse(DSResponse_UploadImage(request).response)



from django.urls import path

from isc_common.views import edited_by

urlpatterns = [

    path('Edited_by/Fetch/', edited_by.Edited_by_Fetch),
    path('Edited_by/Add', edited_by.Edited_by_Add),
    path('Edited_by/Update', edited_by.Edited_by_Update),
    path('Edited_by/Remove', edited_by.Edited_by_Remove),
    path('Edited_by/Lookup/', edited_by.Edited_by_Lookup),
    path('Edited_by/Info/', edited_by.Edited_by_Info),
    path('Edited_by/Copy', edited_by.Edited_by_Copy),

]



# Generated by Django 3.2.6 on 2021-08-23 18:36

from django.db import migrations

import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0118_rename_keyimage_image_types_keyimage'),
    ]

    operations = [
        migrations.AlterField(
            model_name='image_types',
            name='code',
            field=isc_common.fields.code_field.CodeField(blank=True, null=True),
        ),
    ]



# Generated by Django 3.1.6 on 2021-03-01 06:10

import django.db.models.deletion
from django.db import migrations, models

import isc_common.auth.managers.user_manager


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0071_delete_auditmodelwitholdid'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserView',
            fields=[
                ('user_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='isc_common.user')),
            ],
            options={
                'verbose_name': 'Пользователь',
                'db_table': 'isc_common_user_view',
                'managed': False,
            },
            bases=('isc_common.user',),
            managers=[
                ('objects', isc_common.auth.managers.user_manager.UserManager()),
            ],
        ),
    ]



import logging

from django.db.models import CheckConstraint , F , Q
from django.db.models import PositiveIntegerField
from django.forms import model_to_dict
from isc_common.fields.code_field import CodeStrictField
from isc_common.http.DSRequest import DSRequest
from isc_common.models.base_ref import BaseRefHierarcy , BaseRefManager , BaseRefQuerySet
from django.db.models import CheckConstraint , F , Q

logger = logging.getLogger( __name__ )


class Jasper_reportsQuerySet( BaseRefQuerySet ) :
    pass


def get_reports( request ) :
    from reports.models.jasper_reports_users import Jasper_reports_users

    request = DSRequest( request=request )
    jasper_reports = [ dict(
        report=model_to_dict( item.report ) ,
        editor_identifier=item.editor_identifier ,
    ) for item in Jasper_reports_users.objects.filter( user=request.user ) ]
    return jasper_reports


class Jasper_reportsManager( BaseRefManager ) :

    @classmethod
    def getRecord( cls , record ) :
        res = {
            'id' : record.id ,
            'code' : record.code ,
            'name' : record.name ,
            'path' : record.path ,
            'description' : record.description ,
            'username' : record.username ,
            'password' : record.password ,
            'report_id' : record.report_id ,
            'host' : record.host ,
            'port' : record.port ,
            'parent' : record.parent.id if record.parent else None
        }
        return res

    def get_queryset( self ) :
        return Jasper_reportsQuerySet( self.model , using=self._db )


class Jasper_reports( BaseRefHierarcy ) :
    path = CodeStrictField( default='/jasperserver/rest_v2/reports/' )
    report_id = CodeStrictField( null=True , blank=False )
    username = CodeStrictField( default='printer' )
    password = CodeStrictField( default='printer' )
    host = CodeStrictField()
    port = PositiveIntegerField()

    objects = Jasper_reportsManager()

    def __str__( self ) :
        return f"ID:{self.id}, code: {self.code}, name: {self.name}, path: {self.path}, description: {self.description}"

    class Meta :
        verbose_name = 'Отчеты'
        constraints = [
            CheckConstraint( check=~Q( id=F( 'parent_id' ) ) , name=f'c_Jasper_reports' ) ,
        ]



# Generated by Django 3.1.6 on 2021-03-02 16:59

from django.db import migrations

import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0074_auto_20210301_1105'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='e_mails',
            name='phone',
        ),
        migrations.AddField(
            model_name='e_mails',
            name='e_mail',
            field=isc_common.fields.code_field.CodeStrictField(unique=True),
        ),
        migrations.AlterField(
            model_name='phones',
            name='phone',
            field=isc_common.fields.code_field.CodeStrictField(unique=True),
        ),
    ]



import logging
import os
import shutil
import zipfile

from Cryptodome.Cipher import AES
from django.conf import settings
from isc_common.common import SFTP, FILES_STORE_UNKNOWN_TYPE
from isc_common.json import StrToJson
from isc_common.number import TryToInt
from isc_common.oss.functions import get_tmp_dir


class UploadItem:
    id = None
    file_format = None
    file_mime_type = None
    file_size = None
    file_name = None
    real_file_name = None
    tmp_file_name = None
    stored_file_name = None

    code = None
    description = None
    date = None
    date_sign = None
    vatdescr = None
    status_id = None
    precent_type_id = None
    precent_item_type_id = None
    type_id = None

    @property
    def full_path(self):
        if isinstance(settings.FILES_STORE, str):
            return f'{get_tmp_dir(os.path.abspath(settings.FILES_STORE))}{os.sep}{self.stored_file_name}' if self.stored_file_name else None
        else:
            return f'{get_tmp_dir()}{os.sep}{self.stored_file_name}' if self.stored_file_name else None

    def getKey(self):
        return self.key.hex().upper()

    # Зашифровать
    def encript(self, source_path=None):
        if self.full_path:
            if source_path:
                if not os.path.exists(source_path):
                    raise Exception(f'Заданый source_path : {source_path} не существует.')

            cipher = AES.new(self.key, AES.MODE_EAX)
            src = ''
            if (source_path):
                src = f'{source_path}{os.sep}{self.stored_file_name}.'
                self.logger.debug(f'Начало копирования {src} -> {self.full_path}.')
                shutil.copy2(src, self.full_path)
                self.logger.debug(f'Копирование завершено {src} -> {self.full_path}.')

            self.logger.debug(f'Начало чтения {src}')
            data = open(self.full_path, 'rb').read()
            ciphertext, tag = cipher.encrypt_and_digest(data)
            self.logger.debug(f'Чтение {src} завершено.')

            self.logger.debug(f'Начало шифрования {src}')
            file_out = open(f'{self.full_path}', 'wb')
            [file_out.write(x) for x in (cipher.nonce, tag, ciphertext)]
            self.logger.debug(f'Шифрование {src} завершено.')
            return self.key
        else:
            return None

    # Расшифровать
    def decrypt(self):
        if self.full_path:
            file_in = open(self.full_path, 'rb')

            nonce, tag, ciphertext = [file_in.read(x) for x in (16, 16, -1)]

            # let's assume that the key is somehow available again
            cipher = AES.new(self.key, AES.MODE_EAX, nonce)
            data = cipher.decrypt_and_verify(ciphertext, tag)
            path = f'{self.full_path}fordownload'
            res = open(path, 'wb')
            res.write(data)
            res.close()
            return path

    def zip(self):
        z = zipfile.PyZipFile(f'{self.full_path}.zip', 'w', zipfile.ZIP_DEFLATED, True, 2)
        z.write(self.full_path)
        z.close()

    def __init__(self, stored_file_name=None, key=None, logger=None, dictionary=None):
        if logger:
            self.logger = logger
        else:
            self.logger = logging.getLogger(__name__)

        if not settings.FILES_STORE:
            raise Exception('Не задан FILES_STORE.')

        if isinstance(settings.FILES_STORE, str):
            if not os.path.exists(settings.FILES_STORE):
                raise Exception(f'Заданный FILES_STORE : {settings.FILES_STORE} не существует.')
        elif isinstance(settings.FILES_STORE, dict) and settings.FILES_STORE.get('mode') == SFTP:
            if settings.FILES_STORE.get('PATH') is None:
                raise Exception('FILES_STORE не содержит параметр PATH')

            PATH = settings.FILES_STORE.get('PATH')
            res = settings.SSH_CLIENTS.client(settings.FILES_STORE).exists(PATH)

            if res is False:
                raise Exception(f'Заданный FILES_STORE (SFTP) : {PATH} не существует.')

        else:
            raise Exception(FILES_STORE_UNKNOWN_TYPE)

        if dictionary is not None and dictionary.get('key') is not None:
            self.key = dictionary.get('key')
        else:
            self.key = key

        if dictionary is not None and dictionary.get('stored_file_name') is not None:
            self.stored_file_name = dictionary.get('stored_file_name')
        else:
            self.stored_file_name = stored_file_name

        if isinstance(dictionary, str):
            dictionary = StrToJson(dictionary)

        if isinstance(dictionary, dict):
            for k, v in dictionary.items():
                setattr(self, k, TryToInt(v))



# Generated by Django 3.2.6 on 2021-09-01 06:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0126_auto_20210831_2239'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='images',
            name='image_type',
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-22 22:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0024_auto_20190822_1100'),
    ]

    operations = [
        migrations.AddField(
            model_name='shift_day',
            name='dinnertime',
            field=models.TimeField(blank=True, db_index=True, null=True, verbose_name='Конец'),
        ),
    ]



import os
import sys
from os import walk
from pathlib import Path
from shutil import copyfile


def git_push(dest_path):
    print(f'cd {dest_path} && git pull && git add . && git commit -am"." && git push')
    os.system(f'cd {dest_path} && git pull && git add . && git commit -am"." && git push')


def file_append(size, sized, full_path_source, max_size, files, dir_path_dest, full_path_dest):
    if sized + size > max_size:
        if sized > 0:
            return False, sized
    files.append((full_path_source, dir_path_dest, full_path_dest, size))
    sized += size
    print(f'file append ({round(sized / max_size * 1024, 2)} MB) : {full_path_dest}')
    return True, sized


def do(argv):
    # print(argv)
    # print(os.path.abspath(os.curdir))

    max_size = 1024 * 1024 * 1024
    source_path = argv[1]
    dest_path = argv[2]

    if len(argv) < 2 or not os.path.isdir(source_path):
        raise Exception('The path must contain the source directory')

    if len(argv) < 3 or not os.path.isdir(dest_path):
        raise Exception('The path must contain the destination directory')

    excl = ['.git', '.idea', 'TmpFile1', 'Programs', 'tmp', '.php', 'templates', 'sql-log']

    files = []
    sized = 0
    apend_res = False

    for (dirpath, dirnames, filenames) in walk(source_path):
        for filename in filenames:
            full_path_source = f'{dirpath}{os.sep}{filename}'
            exclude = len(list(filter(lambda x: dirpath.find(x) != -1, excl))) > 0
            if exclude is False:
                exclude = len(list(filter(lambda x: full_path_source.find(x) != -1, excl))) > 0
            if exclude is True:
                continue
            stat_source = Path(full_path_source).stat()

            full_path_dest = full_path_source.replace(source_path, dest_path)
            dir_path_dest = dirpath.replace(source_path, dest_path)
            if not os.path.exists(full_path_dest):
                apend_res, sized = file_append(
                    dir_path_dest=dir_path_dest,
                    files=files,
                    full_path_dest=full_path_dest,
                    full_path_source=full_path_source,
                    max_size=max_size,
                    sized=sized,
                    size=stat_source.st_size,
                )
            else:
                stat_dest = Path(full_path_dest).stat()
                if stat_source.st_size != stat_dest.st_size:
                    apend_res, sized = file_append(
                        dir_path_dest=dir_path_dest,
                        files=files,
                        full_path_dest=full_path_dest,
                        full_path_source=full_path_source,
                        max_size=max_size,
                        sized=sized,
                        size=stat_source.st_size,
                    )
                else:
                    print(f'file ({round(sized / max_size * 1024, 2)} MB) : {full_path_dest} exists.')
                    apend_res = True
            if apend_res is False:
                break
        if apend_res is False:
            break

    i = 0
    for src, dir_dst, dst, size in files:
        if not os.path.exists(dir_dst):
            os.makedirs(dir_dst)

        try:
            copyfile(src, dst)
            print(f'Copied ({i} from {len(files)}) {src} -> {dst}')
            i += 1
            sized += size
        except FileNotFoundError as ex:
            print(ex)

    if sized > 0:
        git_push(dest_path=dest_path)

    return apend_res


if __name__ == '__main__':
    a = False
    while a is False:
        a = do(sys.argv)
    print('Done.')



# Generated by Django 3.1.7 on 2021-03-10 04:43

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('isc_common', '0079_auto_20210310_0006'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
        migrations.AddField(
            model_name='user',
            name='deliting',
            field=models.BooleanField(default=True, verbose_name='Возможность удаления'),
        ),
        migrations.AddField(
            model_name='user',
            name='editing',
            field=models.BooleanField(default=True, verbose_name='Возможность редактирования'),
        ),
        migrations.AddField(
            model_name='user',
            name='lastmodified',
            field=models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление'),
        ),
    ]



import logging

from bitfield import BitField

from isc_common.auth.models.user import User
from isc_common.fields.related import ForeignKeyProtect , ForeignKeyCascade
from isc_common.models.audit import Model_withOldIds
from isc_common.models.image_types import Image_types
from isc_common.models.images import Images
from isc_common.models.model_images import Model_imagesQuerySet , Model_images , Model_imagesManager

logger = logging.getLogger(__name__)


class Users_imagesQuerySet(Model_imagesQuerySet):
    pass


class Users_imagesManager(Model_imagesManager):

    @classmethod
    def props(cls):
        return BitField(flags=(
            ('main_photo', 'main_photo'),  # 1
        ), default=0, db_index=True)

    @classmethod
    def getRecord(cls, record ) :
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Users_imagesQuerySet(self.model, using=self._db)


class Users_images(Model_images, Model_withOldIds):
    image = ForeignKeyProtect(Images)
    main_model = ForeignKeyCascade(User)
    props = Users_imagesManager.props()
    type = ForeignKeyProtect(Image_types)

    objects = Users_imagesManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Кросс таблица'
        unique_together = (('image', 'main_model', 'props', 'type', 'deleted_at'),)



from django.urls import path

from events.views import event_types

urlpatterns = [

    path('Event_types/Fetch/', event_types.Event_types_Fetch),
    path('Event_types/Add', event_types.Event_types_Add),
    path('Event_types/Update', event_types.Event_types_Update),
    path('Event_types/Remove', event_types.Event_types_Remove),
    path('Event_types/Lookup/', event_types.Event_types_Lookup),
    path('Event_types/Info/', event_types.Event_types_Info),
    path('Event_types/Copy', event_types.Event_types_Copy),

]



# Generated by Django 3.1.5 on 2021-01-21 06:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0056_last_used_processes'),
    ]

    operations = [
        migrations.CreateModel(
            name='Debug',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField()),
            ],
            options={
                'verbose_name': 'Отладочная информация',
            },
        ),
    ]



from decimal import Decimal
from functools import reduce
from itertools import chain
from uuid import UUID

from bitfield import BitHandler , Bit
from django.db.models import ForeignKey
from isc_common import setAttr


def del_last_zero( value ) :
    if isinstance( value , str ) :
        return value

    res = str( value )
    # res.split('.')
    cel = res.split( '.' )[ 0 ]
    float = res.split( '.' )[ 1 :4 ][ 0 ].rstrip( '0' )
    float = StrToNumber( float )
    if isinstance( float , int ) and float > 0 :
        return f'{cel}.{float}'
    return cel


def DecimalToStr( value , noneId_0=False ) :
    if value is None :
        return ''

    try :
        if value == 0 and noneId_0 is True :
            return None

        res = del_last_zero( value=value )
        return res
    except IndexError :
        return value


def StrToNumber( s1 , default_blank=0 ) :
    if s1 is None :
        return None

    if s1 == '' :
        return default_blank

    if not isinstance( s1 , str ) :
        raise Exception( f'{s1} is not str type.' )

    s = s1.replace( ',' , '.' )

    if not isinstance( s , str ) :
        raise Exception( f'{s} is not str type.' )
    try :
        return int( s )
    except ValueError :
        try :
            return float( s )
        except ValueError as ex :
            return None


def IntToNumber( s ) :
    if s is None :
        return None

    if not isinstance( s , int ) :
        raise Exception( f'{s} is not int type.' )

    return float( s )


def IntToStr( value ) :
    if value is None :
        return None

    if not isinstance( value , str ) :
        return value

    if not isinstance( value , int ) :
        return del_last_zero( value=value )

    raise Exception( f'{value}  must be int' )


def ToStr( s ) :
    if s is None or s == 'null' :
        return None

    if isinstance( s , str ) :
        return s

    if isinstance( s , Decimal ) :
        return DecimalToStr( s )

    if isinstance( s , int ) :
        return IntToStr(s)

    if isinstance( s , UUID ) :
        return str( s ).replace( '-' , '_' )

    return s


def ToNumber( s ) :
    if s is None :
        return 0

    if isinstance( s , int ) :
        return IntToNumber( s )
    elif isinstance( s , str ) :
        return StrToNumber( s )
    elif isinstance( s , float ) :
        return s
    elif isinstance( s , Decimal ) :
        return float( s )
    else :
        raise Exception( f'{s} unknown type.' )


def ToInt( s ) :
    if s is None :
        return None

    if isinstance( s , int ) :
        return s
    elif isinstance( s , str ) :
        return StrToNumber( s )
    elif isinstance( s , float ) :
        return int( s )
    elif isinstance( s , Decimal ) :
        return int( s )
    else :
        raise Exception( f'{s} unknown type.' )


def ToBoolYesNone( value ) :
    if not isinstance( value , int ) :
        return '--'
    if value == 0 :
        return 'Нет'
    elif value == 1 :
        return 'Да'
    else :
        return '--'


def GetListFromStringCortege( stingCortege ) :
    if stingCortege is None :
        return [ None , None ]
    return list( map( lambda x : TryToInt( x ) , stingCortege.replace( '(' , '' ).replace( ')' , '' ).split( ',' ) ) )


def TryToInt( s ) :
    if s is None :
        return None

    try :
        if isinstance( s , int ) :
            return s
        elif isinstance( s , str ) :
            res = StrToNumber( s )
            return res if res is not None else s
        elif isinstance( s , float ) :
            return int( s )
        elif isinstance( s , Decimal ) :
            return int( s )
    except ValueError as ex :
        return s


def ToDecimal( s ) :
    if s is None :
        return 0

    if isinstance( s , int ) :
        return Decimal( s )
    elif isinstance( s , str ) :
        if len( s ) == 0 :
            return 0
        return Decimal( s )
    elif isinstance( s , float ) :
        return Decimal( s )
    elif isinstance( s , Decimal ) :
        return s
    else :
        raise Exception( f'{s} unknown type.' )


def IntToDecimal( s ) :
    if s is None :
        return None

    if not isinstance( s , int ) :
        raise Exception( f'{s} is not int type.' )

    return Decimal( s )


def IntToBool( s ) :
    if s is None :
        return False

    if not isinstance( s , int ) :
        raise Exception( f'{s} is not int type.' )

    if s > 0 :
        return True
    return False


def StrToInt( s ) :
    if s is None :
        return None
    try :
        return int( s )
    except ValueError as ex :
        return None


def DelProps( value ) :
    if isinstance( value , dict ) :
        for key , _value in value.items() :
            if isinstance( _value , BitHandler ) :
                setAttr( value , key , _value._value )
            elif isinstance( _value , Bit ) :
                setAttr( value , key , _value.is_set )
        return value
    else :
        value


def GetPropsInt( value ) :
    if isinstance( value , BitHandler ) :
        return value._value
    else :
        value


def model_2_dict( instance , fields=None , exclude=None ) :
    """
    Return a dict containing the data in ``instance`` suitable for passing as
    a Form's ``initial`` keyword argument.

    ``fields`` is an optional list of field names. If provided, return only the
    named.

    ``exclude`` is an optional list of field names. If provided, exclude the
    named from the returned dict, even if they are listed in the ``fields``
    argument.
    """
    opts = instance._meta
    data = { }
    for f in chain( opts.concrete_fields , opts.private_fields , opts.many_to_many ) :
        if not getattr( f , 'editable' , False ) :
            continue
        if fields is not None and f.name not in fields :
            continue
        if exclude and f.name in exclude :
            continue
        if isinstance( f , ForeignKey ) :
            data[ f'{f.name}_id' ] = f.value_from_object( instance )
        else :
            data[ f.name ] = f.value_from_object( instance )
    return data


def compare_2_dict( dict1 , dict2 ) :
    if not isinstance( dict1 , dict ) :
        raise Exception( f'dict1 must be dict' )

    if not isinstance( dict2 , dict ) :
        raise Exception( f'dict2 must be dict' )

    messages = [ ]

    for x_values , y_values in zip( dict1.items() , dict2.items() ) :
        if x_values != y_values :
            messages.append( f'{x_values} != {y_values}' )
    return messages


def flen( iterable ) :
    return reduce( lambda sum , element : sum + 1 , iterable , 0 )


class Set :
    _lst = [ ]
    _set = set()

    def __init__( self , lst=set() ) :
        if isinstance( lst , set ) :
            self._set.clear()
            for l in list( lst ) :
                self._set.add( l )

        if isinstance( lst , list ) :
            self._lst = lst

    def __str__( self ) :
        if self._set is not None :
            _lst = list( self._set )
            _lst = [ str( l ) for l in _lst ]

            _str = ' , '.join( _lst )
            return f'''({_str})'''
        elif self._lst is not None :
            _lst = [ str( l ) for l in self._lst ]
            _str = ' , '.join( _lst )
            return f'''({_str})'''
        else :
            return '()'

    def is_exists( self , element ) :
        _len = len( self._set )
        self._set.add( element )
        return _len == len( self._set )

    @property
    def get_set_sorted_as_original( self ) :
        if self._lst is not None :
            return sorted( set( self._lst ) , key=self._lst.index )
        elif self._set is not None :
            return list( self._set )
        else :
            return [ ]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from isc_common.models.params import Params, ParamsManager


@JsonResponseWithException()
def Params_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Params.objects.
                filter().
                get_range_rows1(
                request=request,
                function=ParamsManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Params_Add(request):
    return JsonResponse(DSResponseAdd(data=Params.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException(printing=False)
def Params_Update(request):
    return JsonResponse(DSResponseUpdate(data=Params.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException(printing=False)
def Params_Update1(request):
    return JsonResponse(DSResponseUpdate(data=Params.objects.update1FromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Params_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Params.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Params_Remove1(request):
    return JsonResponse(DSResponse(request=request, data=Params.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Params_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Params.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Params_Info(request):
    return JsonResponse(DSResponse(request=request, data=Params.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Params_Get(request):
    return JsonResponse(DSResponse(request=request, data=Params.objects.get_queryset().get_params(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Params_GetKeyValue(request):
    return JsonResponse(DSResponse(request=request, data=Params.objects.get_queryset().get_params_key(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 3.1.7 on 2021-03-04 14:35

import django.db.models.deletion
from django.conf import settings
from django.db import migrations

import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0075_auto_20210302_1659'),
    ]

    operations = [
        migrations.AddField(
            model_name='auditmodelex',
            name='updator',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='AuditModelEx_updator', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='auditmodelex',
            name='creator',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='AuditModelEx_creator', to=settings.AUTH_USER_MODEL),
        ),
    ]



# Generated by Django 3.2.6 on 2021-08-23 07:21

import django.db.models.deletion
from django.db import migrations

import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0105_alter_images_real_name'),
    ]

    operations = [
        migrations.AddField(
            model_name='users_images',
            name='type',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='isc_common.image_types'),
        ),
        migrations.AddField(
            model_name='users_old_images',
            name='type',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='isc_common.image_types'),
        ),
    ]



# Generated by Django 3.2.5 on 2021-07-27 16:31

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0102_alter_images_size'),
        ('react', '0006_auto_20210726_2125'),
    ]

    operations = [
        migrations.AddField(
            model_name='fragment_params',
            name='image',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='isc_common.images'),
        ),
        migrations.AddField(
            model_name='fragment_params',
            name='value_dec',
            field=models.DecimalField(blank=True, decimal_places=4, max_digits=19, null=True),
        ),
        migrations.AddField(
            model_name='fragment_params',
            name='value_text',
            field=models.TextField(blank=True, null=True),
        ),
    ]



from django.urls import path

from isc_common.views import links

urlpatterns = [

    path('Links/Fetch/', links.Links_Fetch),
    path('Links/Add', links.Links_Add),
    path('Links/Update', links.Links_Update),
    path('Links/Remove', links.Links_Remove),
    path('Links/Lookup/', links.Links_Lookup),
    path('Links/Info/', links.Links_Info),
    path('Links/Copy', links.Links_Copy),

]



# Generated by Django 3.2.6 on 2021-08-23 12:38

from django.db import migrations , transaction , connection

from isc_common.models.images import Images
from lfl_admin.common.models.all_tables import All_tables


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0110_alter_images_unique_together'),
    ]

    operations = [
    ]



from django.urls import path

from react.views import page_level_types

urlpatterns = [

    path('Page_level_types/Fetch/', page_level_types.Page_level_types_Fetch),
    path('Page_level_types/Add', page_level_types.Page_level_types_Add),
    path('Page_level_types/Update', page_level_types.Page_level_types_Update),
    path('Page_level_types/Remove', page_level_types.Page_level_types_Remove),
    path('Page_level_types/Lookup/', page_level_types.Page_level_types_Lookup),
    path('Page_level_types/Info/', page_level_types.Page_level_types_Info),
    path('Page_level_types/Copy', page_level_types.Page_level_types_Copy),

]



# Generated by Django 2.2.2 on 2019-06-10 15:33

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('one_c', '0005_auto_20190610_1530'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='documents_param_1c',
            unique_together=set(),
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-25 10:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0061_auto_20190825_1019'),
    ]

    operations = [
        migrations.AlterField(
            model_name='calendar_shifts_days',
            name='shiftorder',
            field=models.PositiveIntegerField(),
        ),
    ]



from django.db import migrations

from isc_common.auth.models.user import User
from isc_common.auth.models.usergroup import UserGroup


def rec_admin_group(apps, schema_editor):
    UserGroup.objects.get_or_create(code='administrators', name="Администраторы", editing=False)
    developers = UserGroup.objects.get_or_create(code='developers', name="Разработчики", editing=False)
    User.objects.create_superuser(username='admin', password='admin', email='info@ivc-inform.ru')
    User.objects.create_user(usergroup=[developers[0]], username='uandrew', password='Uandrew1965', email='uag@ivc-inform.ru')
    User.objects.create_user(usergroup=[developers[0]], username='nasonov', password='nasonov', email='nma@ivc-inform.ru')
    User.objects.create_user(usergroup=[developers[0]], username='developer', password='developer', email='info@ivc-inform.ru', deliting=False, editing=False)

def item_maker(apps, schema_editor):
    group = UserGroup.objects.get_or_create(code='item_maker', name="item_maker", editing=False)
    User.objects.create_user(usergroup=[group[0]], username='item_maker', password='item_maker', email='item_maker@ivc-inform.ru', deliting=False, editing=False)

def rec_message_states(apps, schema_editor):
    from tracker.models.messages_state import Messages_state
    Messages_state.objects.get_or_create(code="new", name="Новая", editing=False, deliting=False)
    Messages_state.objects.get_or_create(code="closed", name="Закрыто")
    Messages_state.objects.get_or_create(code="postponed", name="Отложено")
    Messages_state.objects.get_or_create(code="onworking", name="В работе")
    Messages_state.objects.get_or_create(code="wait", name="Ожидание")

def rec_message_theme(apps, schema_editor):
    from tracker.models.messages_theme import Messages_theme
    Messages_theme.objects.get_or_create(code="auto_from_error", name="Автоматически занесенные из сообщений об ошибках.", editing=False, deliting=False)

def rec_message_theme1(apps, schema_editor):
    from tracker.models.messages_theme import Messages_theme
    Messages_theme.objects.get_or_create(code="item_maker", name="Процесс заполнения товареных позиций.", editing=False, deliting=False)


class Migration(migrations.Migration):
    dependencies = [
        ('isc_common', '0022_auto_20190414_0925'),
    ]
    atomic = True

    operations = [
        migrations.RunPython(rec_admin_group),
        migrations.RunPython(item_maker),
        migrations.RunPython(rec_message_states),
        migrations.RunPython(rec_message_theme),
        migrations.RunPython(rec_message_theme1),
    ]



# Generated by Django 3.2.6 on 2021-08-23 12:46

from django.db import migrations , transaction
from tqdm import tqdm

from isc_common.common import unknown
from lfl_admin.competitions.models.disqualification_condition import Disqualification_condition
from lfl_admin.competitions.models.disqualification_zones import Disqualification_zones
from lfl_admin.competitions.models.division_stages import Division_stages
from lfl_admin.competitions.models.divisions import Divisions
from lfl_admin.competitions.models.tournaments import Tournaments
from lfl_admin.region.models.regions import Regions


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0111_auto_20210823_1238'),
    ]

    operations = [

    ]



# Generated by Django 3.2.7 on 2021-09-16 12:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0132_alter_images_unique_together'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='usergroup_permission',
            unique_together=None,
        ),
        migrations.RemoveField(
            model_name='usergroup_permission',
            name='usergroup',
        ),
        migrations.RemoveField(
            model_name='usergroup_permission',
            name='widget',
        ),
        migrations.RemoveField(
            model_name='widgets_trees',
            name='parent',
        ),
        migrations.AlterModelOptions(
            name='usergroup',
            options={'verbose_name': 'группа'},
        ),
        migrations.DeleteModel(
            name='User_Permission',
        ),
        migrations.DeleteModel(
            name='Usergroup_permission',
        ),
        migrations.DeleteModel(
            name='Widgets_trees',
        ),
    ]



# Generated by Django 3.2.4 on 2021-07-26 21:25

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('react', '0005_alter_fragments_item_types_options'),
    ]

    operations = [
        migrations.AddField(
            model_name='fragment_param_types',
            name='code',
            field=isc_common.fields.code_field.CodeStrictField(unique=True),
        ),
        migrations.AddField(
            model_name='fragment_param_types',
            name='description',
            field=isc_common.fields.description_field.DescriptionField(),
        ),
        migrations.AddField(
            model_name='fragment_param_types',
            name='name',
            field=isc_common.fields.name_field.NameField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='fragment_params',
            name='code',
            field=isc_common.fields.code_field.CodeStrictField(unique=True),
        ),
        migrations.AddField(
            model_name='fragment_params',
            name='description',
            field=isc_common.fields.description_field.DescriptionField(),
        ),
        migrations.AddField(
            model_name='fragment_params',
            name='name',
            field=isc_common.fields.name_field.NameField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='fragments',
            name='parent',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='react.fragments'),
        ),
        migrations.AddField(
            model_name='fragments_item_types',
            name='code',
            field=isc_common.fields.code_field.CodeStrictField(unique=True),
        ),
        migrations.AddField(
            model_name='fragments_item_types',
            name='description',
            field=isc_common.fields.description_field.DescriptionField(),
        ),
        migrations.AddField(
            model_name='fragments_item_types',
            name='name',
            field=isc_common.fields.name_field.NameField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='fragments',
            name='code',
            field=isc_common.fields.code_field.CodeStrictField(),
        ),
    ]



import logging
import os
from os.path import expanduser

from django.conf import settings
from django.db import transaction
from isc_common import Stack
from isc_common.fields.files import make_REPLACE_FILE_PATH , make_REPLACE_IMAGE_PATH
from isc_common.models.audit import AuditModel , AuditManager , AuditQuerySet
from isc_common.number import flen

logger = logging.getLogger( __name__ )


class Model_imagesQuerySet( AuditQuerySet ) :
    pass


class Model_imagesManager( AuditManager ) :

    @classmethod
    def getRecord( cls , record ) :
        res = {
            'id' : record.id ,
        }
        return res

    def get_queryset( self ) :
        return Model_imagesQuerySet( self.model , using=self._db )


class ImageNotFound( Exception ) :
    pass


class Model_images( AuditModel ) :
    @classmethod
    def get_image( cls , file_name , path=None ) :
        if file_name is None :
            return None

        if os.altsep is not None :
            have_path = file_name.find( os.altsep ) != -1
        else :
            have_path = file_name.find( os.sep ) != -1

        if have_path :
            if os.altsep is None :
                file_name1 = f'{settings.OLD_SITE_BASE_DIR}{file_name}'.replace( os.sep , '\\' )
            else :
                file_name1 = f'{settings.OLD_SITE_BASE_DIR}{file_name}'.replace( os.altsep , os.sep )
            try :
                return cls.objects.get( path__upper=file_name1.upper() ).path
            except cls.DoesNotExist :
                return None

        query = cls.objects.filter( file_name=file_name ).order_by( '-date' )
        if query.count() == 0 :
            return None
        else :
            _list = list( map( lambda x : x.path , query ) )
            if path is None :
                return _list[ 0 ]

            for item in _list :
                if item.find( path ) != -1 :
                    return item
            return None

    @classmethod
    def rec_update( cls , main_model , image , code , keyimage , result=None , self_image_field=False , defaults=None ) :
        if self_image_field is False :
            from isc_common.models.image_types import Image_types
            image_type , _ = Image_types.objects.get_or_create( code=code , keyimage=keyimage )

            if isinstance( main_model , int ) :
                res , _ = cls.objects.update_or_create( image=image , type=image_type , main_model_id=main_model , defaults=defaults )
            else :
                res , _ = cls.objects.update_or_create( image=image , type=image_type , main_model=main_model , defaults=defaults )

            if result is not None :
                result.push( res , lambda item , stack : flen( filter( lambda x : x.id == res.id , stack ) ) > 0 )
        else :
            raise Exception( 'Not implementation' )
            # eval( f'main_model._meta.concrete_model.objects.filter({pk_name}=main_model.{pk_name}).update({image_field_name}=image)' , dict() , dict( image=image , main_model=main_model ) )

    @classmethod
    def update_or_create_image( cls , main_model , path , file_name , keyimage , self_image_field=False , code=None , exception=True , defaults=None, o_ssh_client=None ) :
        from isc_common.models.images import Images
        from lfl_admin.common.models.site_lfl_images import Site_lfl_images

        result = Stack()

        if file_name is None or file_name.strip() == '' :
            return result

        if (code is None or code.strip() == '') or (path is None or path.strip() == '') :
            code = path

        with transaction.atomic() :
            full_name_images = Site_lfl_images.get_image( file_name=file_name , path=path )

            if len( full_name_images ) == 0 and file_name is not None :
                if exception is True :
                    raise ImageNotFound( f'{file_name} : not found' )
                # else :
                #     logger.debug( f'{file_name} : not found !!!' )

            for full_name_image in full_name_images :
                if full_name_image is not None :

                    zero = False

                    full_name_image_replaced = make_REPLACE_IMAGE_PATH( full_name_image )
                    home = expanduser( "~" )
                    localpath = f"{home}/tmp"
                    if not os.path.exists( localpath ) :
                        os.makedirs( localpath )

                    if o_ssh_client is None:
                        o_ssh_client = settings.SSH_CLIENTS.client( settings.OLD_FILES )

                    if o_ssh_client.exists( full_name_image_replaced ) :
                        size = o_ssh_client.getsize( full_name_image_replaced )
                        if size == 0 :
                            image.real_name = full_name_image_replaced
                            image.size = 0
                            image.attfile = ''
                            image.save()
                            continue

                        _ , _file_name = os.path.split( full_name_image_replaced )
                        # logger.debug( f'file_name: {_file_name}, size: {size}' )
                        image = Images.objects.getOptional( file_name=_file_name , size=size )
                        if image is None :
                            image = Images.objects.getOptional( real_name=full_name_image_replaced , size=size )

                            if (image is not None) :
                                image.file_name = _file_name
                                image.size = size
                                image.save()
                            else :
                                image = Images.objects.create( real_name=full_name_image_replaced , file_name=_file_name , size=size )

                    else :
                        zero = True
                        logger.debug( f'{full_name_image_replaced} : not found' )

                    if not zero :
                        cls.rec_update(
                            code=code ,
                            defaults=defaults ,
                            image=image ,
                            keyimage=keyimage ,
                            main_model=main_model ,
                            result=result ,
                            self_image_field=self_image_field ,
                        )
                        # logger.debug( f'{file_name} : found' )

            return result

    @classmethod
    def replace_image( cls , image , _image , django_model_images , imports , main_model , keyimage ) :
        for _import in imports :
            eval( f'exec("{_import}")' )

        s4compile = f'''{django_model_images}.objects.filter(image_id={image.id}).update(image_id={_image.id})'''
        code = compile( s4compile , '<string>' , 'eval' )
        eval( code )
        cls.rec_update(
            code=code ,
            image=_image ,
            main_model=main_model ,
            keyimage=keyimage ,
        )

    @classmethod
    def update_or_create_image1( cls , main_model , code , path , keyimage , image_id , django_model_images , imports , defaults=None ) :
        from isc_common.models.images import Images

        imports = imports.split( ',' )

        image1 = None
        if image_id is None :
            return None

        with transaction.atomic() :
            image = Images.objects.get( id=image_id )
            full_name_image = image.real_name
            if full_name_image is not None :

                full_name_image_replaced = make_REPLACE_IMAGE_PATH( full_name_image )

                home = expanduser( "~" )
                localpath = f"{home}/tmp"
                if not os.path.exists( localpath ) :
                    os.makedirs( localpath )

                o_ssh_clients = settings.SSH_CLIENTS.client( settings.OLD_FILES )
                if o_ssh_clients.exists( full_name_image_replaced ) :
                    size = o_ssh_clients.getsize( full_name_image_replaced )
                    _ , fileName = os.path.split( full_name_image_replaced )

                    _image = Images.objects.getOptional( real_name=full_name_image_replaced , size=size )
                    if _image is None :
                        _image = Images.objects.getOptional( real_name=fileName , size=size )
                    else :
                        _image.real_name = fileName;
                        _image.save()

                    if _image is not None :

                        if str( _image.attfile ) != '' and settings.SSH_CLIENTS.client( settings.FILES_STORE ).exists( make_REPLACE_FILE_PATH( str( _image.attfile ) ) ) :
                            cls.replace_image(
                                image=image ,
                                _image=_image ,
                                django_model_images=django_model_images ,
                                imports=imports ,
                                main_model=main_model ,
                                keyimage=keyimage ,
                            )
                            return _image.id
                        else :
                            _image.size = 0
                            _image.save()

                    image1 , created = Images.create_update(
                        id=image.id ,
                        real_file_name=full_name_image_replaced ,
                        tmp_file_name=localpath ,
                    )

                    if image1.id != image.id :
                        cls.replace_image(
                            image=image ,
                            _image=image1 ,
                            django_model_images=django_model_images ,
                            imports=imports ,
                            main_model=main_model ,
                            keyimage=keyimage ,
                        )
                        return image1.id
                else :
                    return None

                cls.rec_update(
                    code=code ,
                    keyimage=keyimage ,
                    image=image ,
                    main_model=main_model ,
                    defaults=defaults ,
                )
                return image.id
            return image.id

    objects = Model_imagesManager()

    def __str__( self ) :
        return f'ID:{self.id}'

    def __repr__( self ) :
        return self.__str__()

    class Meta :
        verbose_name = 'Базовый класс'
        abstract = True
